/*******************************************************************************
Copyright (C) Marvell International Ltd. and its affiliates

This software file (the "File") is owned and distributed by Marvell 
International Ltd. and/or its affiliates ("Marvell") under the following
alternative licensing terms.  Once you have made an election to distribute the
File under one of the following license alternatives, please (i) delete this
introductory statement regarding license alternatives, (ii) delete the two
license alternatives that you have not elected to use and (iii) preserve the
Marvell copyright notice above.

********************************************************************************
Marvell Commercial License Option

If you received this File from Marvell and you have entered into a commercial
license agreement (a "Commercial License") with Marvell, the File is licensed
to you under the terms of the applicable Commercial License.

********************************************************************************
Marvell GPL License Option

If you received this File from Marvell, you may opt to use, redistribute and/or 
modify this File in accordance with the terms and conditions of the General 
Public License Version 2, June 1991 (the "GPL License"), a copy of which is 
available along with the File in the license.txt file or by writing to the Free 
Software Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 or 
on the worldwide web at http://www.gnu.org/licenses/gpl.txt. 

THE FILE IS DISTRIBUTED AS-IS, WITHOUT WARRANTY OF ANY KIND, AND THE IMPLIED 
WARRANTIES OF MERCHANTABILITY OR FITNESS FOR A PARTICULAR PURPOSE ARE EXPRESSLY 
DISCLAIMED.  The GPL License provides additional details about this warranty 
disclaimer.
********************************************************************************
Marvell BSD License Option

If you received this File from Marvell, you may opt to use, redistribute and/or 
modify this File under the following licensing terms. 
Redistribution and use in source and binary forms, with or without modification, 
are permitted provided that the following conditions are met:

    *   Redistributions of source code must retain the above copyright notice,
	    this list of conditions and the following disclaimer. 

    *   Redistributions in binary form must reproduce the above copyright
        notice, this list of conditions and the following disclaimer in the
        documentation and/or other materials provided with the distribution. 

    *   Neither the name of Marvell nor the names of its contributors may be 
        used to endorse or promote products derived from this software without 
        specific prior written permission. 
    
THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" AND 
ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED 
WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE 
DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR CONTRIBUTORS BE LIABLE FOR 
ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES 
(INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; 
LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON 
ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT 
(INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS 
SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.

*******************************************************************************/


#include "mvBoardEnvLib.h"
#include "mvCtrlEnvSpec.h"
#include "mvCpuIf.h"
#include "mvCntmr.h"
#include "mvDevice.h"
#include "mvDramIfRegs.h"
#include "mvPciRegs.h"
#include "mvPexRegs.h"
#include "mvTwsi.h"
#include "mvUbootArm.h"

#if defined(CONFIG_BUFFALO_PLATFORM) && !defined(ORG_PCB)
#include "mvUart.h"
#include "mvGpp.h"

#define STATIC_SDRAM_TIME_CTRL_LOW_250MHz	0x11b13330
#define STATIC_SDRAM_TIME_CTRL_LOW_200MHz	0x11812220

struct buffalo_st
{
	MV_U32 pid;
	MV_U32 micon_support;
	MV_U32 rtc_type;
};

struct buffalo_st buffalo_nas_info[]={
/*	PID,			MCON,	RTC			*/
	PID_LS_GL,		1,	0,	//0:LS-GL
	PID_HS_DHGL,		1,	0,	//1:HS-DHGL
	PID_LS_LGL,		0,	0,	//2:LS-LGL
	PID_LS_IGL,		1,	0,	//3:LS-IGL
	PID_TS_HTGL_R5,		1,	0,	//4:TS-HTGL/R5
	PID_HS_DHTGL_R5,	1,	0,	//5:HS-DHTGL/R5
	PID_TS_2UHTGL_R5,	1,	0,	//6:TS-2UHTGL/R5
	PID_HS_WDHTGL_R1,	1,	0,	//7:HS-WDHTGL/R1
	PID_LS_WTGL_R1,		0,	0,	//8:LS-WTGL/R1
	PID_LS_WGL,		0,	0,	//9:LS-WGL
	PID_LS_WSGL_R1,		0,	0,	//0xA:LS-WSGL/R1
	PID_LS_HGL,		0,	0,	//0xB:LG-HGL
	PID_LS_WHGL_R1,		0,	0,	//0xC:LG-WHGL/R1
	PID_LS_WWN_R1,		0,	0,	//0xD:LS-WWN/R1
	PID_LS_CL,		0,	0,	//0xE:LS-CL
	PID_LS_QL_R5,		0,	0,	//0xF:LG-QL/R5
};

MV_U32 bid_to_buffalo_st_index[] = {	N_A, 	//0x00
					N_A, 	//0x01
					N_A, 	//0x02
					N_A, 	//0x03
					N_A,	//0x04
					N_A, 	//0x05
					N_A, 	//0x06
					N_A, 	//0x07
					N_A, 	//0x08
					N_A, 	//0x09
					N_A, 	//0x0a
					N_A, 	//0x0b
					N_A, 	//0x0c
					N_A, 	//0x0d
					N_A, 	//0x0e
					N_A, 	//0x0f
					N_A, 	//0x10
					N_A, 	//0x11
					0, 	//0x12 LS-GL
					N_A, 	//0x13
					4, 	//0x14 TS-HTGL
					N_A, 	//0x15
					N_A, 	//0x16
					7, 	//0x17 HS-WDHTGL
					8, 	//0x18 LS-WTGL
					9, 	//0x19 LS-WGL
					0xa,	//0x1A LS-WSGL/R1
					0xB, 	//0x1B LS-HGL
					0xc, 	//0x1C LS-WHGL/R1
					0xD,	//0x1D LS-WWN/R1
					0xE, 	//0x1E LS-CL
					0xF, 	//0x1F LS-QL/R5
				};

#endif

/* defines  */
#ifdef MV_DEBUG
	#define DB(x)	x
#else
	#define DB(x)
#endif	

MV_STATUS boardEepromGet(BOARD_DATA    *boardData);

MV_U32 gBoardId = -1;
MV_U32 gBuffaloCpuId = -1;
/*******************************************************************************
* mvBoardIdGet - Get Board model
*
* DESCRIPTION:
*       This function returns board ID.
*       Board ID is 32bit word constructed of board model (16bit) and 
*       board revision (16bit) in the following way: 0xMMMMRRRR.
*
* INPUT:
*       None.
*
* OUTPUT:
*       None.
*
* RETURN:
*       32bit board ID number, '-1' if board is undefined.
*
*******************************************************************************/
#if defined(CONFIG_BUFFALO_PLATFORM) && !defined(ORG_PCB)
MV_U32
BuffaloBoardGetStrapStatus(MV_VOID)
{
	char name[15];
	int i=0;
	MV_U32 RegBackup[4];
	MV_U32 StrapStatus = 0x0;
	MV_U32 StrapMask = 0x0;

	MV_U32 CPU_ID = BuffaloGetCpuId();

	switch(CPU_ID)
	{
	case 0x5281:
		StrapMask = STRAP_5281_MASK;
		break;
	case 0x5182:
		StrapMask = STRAP_5182_MASK;
		break;
	default:
		StrapMask = STRAP_5182_MASK;
		break;
	}

	for(i=0; i<4; i++)
	{
		RegBackup[i] = MV_REG_READ(mvMppRegGet(i));
	}

	MV_U32 TMP_MPP_REG[4] = {	0x00000003,
					0x00000000,
					0x00005555,
					0xFFFF0000,
				};
				
	for(i=0; i<4; i++)
	{
		MV_REG_WRITE(mvMppRegGet(i), TMP_MPP_REG[i]);
	}

	mvGppOutEnablle(0, StrapMask, (MV_GPP_IN & StrapMask));
	mvGppPolaritySet(0, StrapMask, (MV_GPP_IN_ORIGIN & StrapMask));
	StrapStatus = (MV_REG_READ(GPP_DATA_IN_REG(0)) & StrapMask);

	for(i=0; i<4; i++)
	{
		MV_REG_WRITE(mvMppRegGet(i), RegBackup[i]);
	}

	return StrapStatus;

	//return mvGppValueGet(0, STRAP_PINS);
}

MV_U32
IsSupportMicon(MV_VOID)
{
	//printf("%s> buffalo_nas_info[bid_to_buffalo_st_index[gboardId]] = 0x%02x\n", __FUNCTION__, buffalo_nas_info[bid_to_buffalo_st_index[gBoardId]]);
	return buffalo_nas_info[bid_to_buffalo_st_index[gBoardId]].micon_support;
}

MV_U32
IsSupportRtcRegWrite(MV_VOID)
{
	switch(buffalo_nas_info[bid_to_buffalo_st_index[gBoardId]].rtc_type){
	case RTC_RICOH:
		return 1;
		break;
	case RTC_EPSON:
		return 0;
		break;
	}
}

MV_U32
BuffaloGetCpuId(MV_VOID)
{
	if (gBuffaloCpuId != -1)
		return gBuffaloCpuId;

	char name[15];
	mvCtrlModelRevNameGet(name);

	if (strncmp(name, "88F5281", strlen("88F5281")) == 0)
		gBuffaloCpuId = 0x5281;
	else if (strncmp(name, "88F5182", strlen("88F5182")) == 0)
		gBuffaloCpuId = 0x5182;
	else
		gBuffaloCpuId = 0x5182;

	return gBuffaloCpuId;
}

MV_U32 mvBoardIdGet(MV_VOID)
{
	MV_U32 tmpBoardId = -1;
	MV_U32 tmpStrapStatus = -1;

	if(gBoardId != -1)
		return gBoardId;

		
	MV_U32 CPU_ID = BuffaloGetCpuId();
	tmpStrapStatus = BuffaloBoardGetStrapStatus();

	switch(CPU_ID)
	{
	case 0x5281:
		switch(tmpStrapStatus)
		{
		case STRAP_STATUS_LSWWN:
			gBoardId = BUFFALO_BOARD_LS_WWN_R1;
			return BUFFALO_BOARD_LS_WWN_R1;
			break;
		default:
			gBoardId = BUFFALO_BOARD_LS_WWN_R1;
			return BUFFALO_BOARD_LS_WWN_R1;
			break;
		}
		break;
	case 0x5182:
		switch(tmpStrapStatus){
		case STRAP_STATUS_LSQL:
			gBoardId = BUFFALO_BOARD_LS_QL_R5;
			return BUFFALO_BOARD_LS_QL_R5;
			break;
		case STRAP_STATUS_LSCL:
			gBoardId = BUFFALO_BOARD_LS_CL;
			return BUFFALO_BOARD_LS_CL;
			break;
		case STRAP_STATUS_LSWHGL:
			gBoardId = BUFFALO_BOARD_LS_WHGL_R1;
			return BUFFALO_BOARD_LS_WHGL_R1;
			break;
		case STRAP_STATUS_LSHGL:
			gBoardId = BUFFALO_BOARD_LS_HGL;
			return BUFFALO_BOARD_LS_HGL;
			break;
		case STRAP_STATUS_LSWSGL:
			gBoardId = BUFFALO_BOARD_LS_WSGL_R1;
			return BUFFALO_BOARD_LS_WSGL_R1;
			break;
		case STRAP_STATUS_LSWGL:
			gBoardId = BUFFALO_BOARD_LS_WGL;
			return BUFFALO_BOARD_LS_GL;
			break;
		case STRAP_STATUS_LSWTGL:
			gBoardId = BUFFALO_BOARD_LS_WTGL_R1;
			return BUFFALO_BOARD_LS_WTGL_R1;
			break;
		case STRAP_STATUS_HSWDHTGL:
			gBoardId = BUFFALO_BOARD_HS_WDHTGL_R1;
			return BUFFALO_BOARD_HS_WDHTGL_R1;
			break;
		case STRAP_STATUS_LSGL_EG:
			gBoardId = BUFFALO_BOARD_LS_GL ;
			return BUFFALO_BOARD_LS_GL ;
			break;
		case STRAP_STATUS_LSGL:
			gBoardId = BUFFALO_BOARD_LS_GL ;
			return BUFFALO_BOARD_LS_GL ;
			break;
		case STRAP_STATUS_KURO:
			gBoardId = BUFFALO_BOARD_LS_GL ;
			return BUFFALO_BOARD_LS_GL ;
			break;
		default :
			gBoardId = BUFFALO_BOARD_LS_WTGL_R1;
			return BUFFALO_BOARD_LS_WTGL_R1;
			break;
		}
		break;
	default:
		break;
	}

#if !defined(CONFIG_BUFFALO_LGL_PLATFORM)
	tmpBoardId = mcon_get_borad_id();

	if(tmpBoardId == 0xf4)
	{
		if (MV_REG_READ(SDRAM_TIMING_CTRL_LOW_REG) == STATIC_SDRAM_TIME_CTRL_LOW_250MHz)
		{
			gBoardId = BUFFALO_BOARD_LS_GL ;
			return BUFFALO_BOARD_LS_GL ;
		}
		else
		{
			gBoardId = BUFFALO_BOARD_LS_GL ;
			return BUFFALO_BOARD_LS_GL ;
		}
	}
	else
	{
	     gBoardId = tmpBoardId ;
		return gBoardId;
	}
/* This is strange code but the program can not use memory not yet. saito  */
#endif

}

#else
MV_U32 mvBoardIdGet(MV_VOID)
{
	MV_U32 tmpBoardId = -1;
	BOARD_DATA    boardData;

	if(gBoardId != -1)
		return gBoardId;

#if defined(MV_88F1181)
	
	if(boardEepromGet(&boardData) == MV_OK)
	{
		tmpBoardId = (MV_U32)boardData.boardId;
	}
	else
	{
		/* until we have relevant data in twsi then we 
		will detect the board type from sdram config reg */
		if (MV_REG_READ(SDRAM_CONFIG_REG) & SDRAM_DTYPE_DDR2)
		{
			tmpBoardId = DB_88F1181_DDR2;
		}
		else
		{
			tmpBoardId = DB_88F1181_DDR1;
		}

	}
	

#elif defined(MV_88F5181)

	if (MV_REG_READ(SDRAM_CONFIG_REG) & SDRAM_DTYPE_DDR2)
	{
	}
	else /* DDR1 */
	{
		#if defined(RD_88F5182)
		tmpBoardId = RD_88F5182_2XSATA;
		#elif defined(RD_88F5182_3)
		tmpBoardId = RD_88F5182_2XSATA3;
		#elif defined(RD_88W8660)
		tmpBoardId = RD_88W8660_DDR1;
		#elif defined(RD_88W8660_AP82S)
		tmpBoardId = RD_88W8660_AP82S_DDR1;
		#elif defined(RD_88F5181L_FE)
		tmpBoardId = RD_88F5181L_VOIP_FE;
		#elif defined(RD_88F5181L_GE)
		tmpBoardId = RD_88F5181L_VOIP_GE;
        #elif defined(MV_POS_NAS) 
        tmpBoardId = RD_88F5181_POS_NAS;
		#elif defined(MV_VOIP)
		tmpBoardId = RD_88F5181_VOIP;
        #elif defined(DB_PRPMC)
		tmpBoardId = DB_88F5181_DDR1_PRPMC;
        	#elif defined(DB_MNG)
		tmpBoardId = DB_88F5181_DDR1_MNG;
		#elif defined(DB_PEX_PCI)
		tmpBoardId = DB_88F5181_DDR1_PEXPCI;
		#endif
	}
	if(tmpBoardId != -1) {
		gBoardId = tmpBoardId;
		return tmpBoardId;
	}


	if(boardEepromGet(&boardData) == MV_OK)
	{
		tmpBoardId = (MV_U32)boardData.boardId;
	}
	else
	{
		/* until we have relevant data in twsi then we 
		will detect the board type from sdram config reg */
		if (MV_REG_READ(SDRAM_CONFIG_REG) & SDRAM_DTYPE_DDR2)
		{
			if((mvCtrlModelGet() == MV_5281_DEV_ID)&&
			   (mvCtrlRevGet() >= MV_5281_B0_REV))
			{
				tmpBoardId = DB_88F5X81_DDR2;
			} 
			else if(mvCtrlModelGet() == MV_8660_DEV_ID)
			{
				tmpBoardId = DB_88W8660_DDR2;
			}
			else if((mvCtrlModelGet() == MV_5182_DEV_ID) || (mvCtrlModelGet() == MV_5082_DEV_ID))
			{
				tmpBoardId = DB_88F5182_DDR2;
			}
			else
			{
				tmpBoardId = DB_88F5181_5281_DDR2;
			}
		}
		else /* DDR1 */
		{
			if (MV_REG_READ(PCI_ARBITER_CTRL_REG(0)) & PACR_ARB_ENABLE) /* arbiter enabled*/
			{
				if((mvCtrlModelGet() == MV_5281_DEV_ID)&&
				   (mvCtrlRevGet() >= MV_5281_B0_REV))
				{
					tmpBoardId = DB_88F5X81_DDR1;
				}
				else
				{
					tmpBoardId = DB_88F5181_5281_DDR1;
				}
			}
			else /* arbiter disabled */
			{

				if ((MV_REG_READ(PEX_CTRL_REG(0)) & PXCR_DEV_TYPE_CTRL_MASK)
					 == PXCR_DEV_TYPE_CTRL_CMPLX) /*root complex*/
				{
					#if defined(DB_PRPMC)
					tmpBoardId = DB_88F5181_DDR1_PRPMC;

        				#elif defined(DB_MNG)
						tmpBoardId = DB_88F5181_DDR1_MNG;
					#endif
				}
				else if ((MV_REG_READ(PEX_CTRL_REG(0)) & PXCR_DEV_TYPE_CTRL_MASK)
					 == PXCR_DEV_TYPE_CTRL_POINT) /*end point*/
				{
					tmpBoardId = DB_88F5181_DDR1_PEXPCI;
				}
			}
		}

	}

	gBoardId = tmpBoardId;

	return tmpBoardId;

#else
#   error "CHIP not selected"
#endif

}
#endif
/* saito change*/
#if defined(CONFIG_BUFFALO_PLATFORM) && !defined(ORG_PCB)
#else

/*******************************************************************************
* boardEepromGet - Get board identification from the EEPROM
*
* DESCRIPTION:
*
* INPUT:
*
* OUTPUT:
*       None.
*
* RETURN:
*
*******************************************************************************/
MV_STATUS boardEepromGet(BOARD_DATA    *boardData)
{
 	MV_TWSI_SLAVE twsiSlave;
        MV_TWSI_ADDR slave;

	MV_U32 tclk;

	tclk = mvBoardTclkGet();
	
	/* Init TWSI first */
	slave.type = ADDR7_BIT;
	slave.address = 0x0;
	mvTwsiInit(100000, tclk, &slave, 0);

   	twsiSlave.slaveAddr.address = MV_BOARD_ID_EEPROM;
	twsiSlave.slaveAddr.type = ADDR7_BIT;
	twsiSlave.validOffset = MV_TRUE;
   	twsiSlave.offset = MV_BOARD_ID_EEPROM_OFFSET0;
	twsiSlave.moreThen256 = MV_FALSE;

	if(MV_OK != mvTwsiRead (&twsiSlave, (MV_U8*)boardData, sizeof(BOARD_DATA)))
    {
		/*mvOsOutput("Fail to read Board EEPROM from offset0");*/
       	return MV_FAIL;
   	}

#if defined(MV_CPU_LE)
	boardData->magic = MV_BYTE_SWAP_32BIT(boardData->magic);
	boardData->boardId = MV_BYTE_SWAP_16BIT(boardData->boardId);
	boardData->reserved1 = MV_BYTE_SWAP_32BIT(boardData->reserved1);
	boardData->reserved2 = MV_BYTE_SWAP_32BIT(boardData->reserved2);
#endif

	if(boardData->magic == MV_BOARD_I2C_MAGIC)
	{
    	return MV_OK;
    }

   	twsiSlave.offset = MV_BOARD_ID_EEPROM_OFFSET1;
	twsiSlave.moreThen256 = MV_TRUE;

	if(MV_OK != mvTwsiRead (&twsiSlave, (MV_U8*)boardData, sizeof(BOARD_DATA)))
    {
		/*mvOsOutput("Fail to read Board EEPROM from offset1");*/
       	return MV_FAIL;
   	}


	
#if defined(MV_CPU_LE)
	boardData->magic = MV_BYTE_SWAP_32BIT(boardData->magic);
	boardData->boardId = MV_BYTE_SWAP_16BIT(boardData->boardId);
	boardData->reserved1 = MV_BYTE_SWAP_32BIT(boardData->reserved1);
	boardData->reserved2 = MV_BYTE_SWAP_32BIT(boardData->reserved2);
#endif
	
    if(boardData->magic == MV_BOARD_I2C_MAGIC)
	{
    	return MV_OK;
    }

	return MV_FAIL;
}
#endif


