/*
 * (C) Copyright 2002
 * Wolfgang Denk, DENX Software Engineering, wd@denx.de.
 *
 * See file CREDITS for list of people who contributed to this
 * project.
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License as
 * published by the Free Software Foundation; either version 2 of
 * the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston,
 * MA 02111-1307 USA
 */

#ifndef _ENVIRONMENT_H_
#define _ENVIRONMENT_H_	1

#define FLAG_SAVE_PRESENT	0
#define FLAG_SAVE_DEFAULT	1

/**************************************************************************
 *
 * The "environment" is stored as a list of '\0' terminated
 * "name=value" strings. The end of the list is marked by a double
 * '\0'. New entries are always added at the end. Deleting an entry
 * shifts the remaining entries to the front. Replacing an entry is a
 * combination of deleting the old value and adding the new one.
 *
 * The environment is preceeded by a 32 bit CRC over the data part.
 *
 **************************************************************************
 */

#if defined(CFG_ENV_IS_IN_FLASH)
# ifndef  CFG_ENV_ADDR
#  define CFG_ENV_ADDR	(CFG_FLASH_BASE + CFG_ENV_OFFSET)
# endif
# ifndef  CFG_ENV_OFFSET
#  define CFG_ENV_OFFSET (CFG_ENV_ADDR - CFG_FLASH_BASE)
# endif
# if !defined(CFG_ENV_ADDR_REDUND) && defined(CFG_ENV_OFFSET_REDUND)
#  define CFG_ENV_ADDR_REDUND	(CFG_FLASH_BASE + CFG_ENV_OFFSET_REDUND)
# endif
# if defined(CFG_ENV_SECT_SIZE) || defined(CFG_ENV_SIZE)
#  ifndef  CFG_ENV_SECT_SIZE
#   define CFG_ENV_SECT_SIZE	CFG_ENV_SIZE
#  endif
#  ifndef  CFG_ENV_SIZE
#   define CFG_ENV_SIZE	CFG_ENV_SECT_SIZE
#  endif
# else
#  error "Both CFG_ENV_SECT_SIZE and CFG_ENV_SIZE undefined"
# endif
# if defined(CFG_ENV_ADDR_REDUND) && !defined(CFG_ENV_SIZE_REDUND)
#  define CFG_ENV_SIZE_REDUND	CFG_ENV_SIZE
# endif
# if (CFG_ENV_ADDR >= CFG_MONITOR_BASE) && \
     (CFG_ENV_ADDR+CFG_ENV_SIZE) <= (CFG_MONITOR_BASE + CFG_MONITOR_LEN)
#  define ENV_IS_EMBEDDED	1
# endif
# if defined(CFG_ENV_ADDR_REDUND) || defined(CFG_ENV_OFFSET_REDUND)
#  define CFG_REDUNDAND_ENVIRONMENT	1
# endif
#endif	/* CFG_ENV_IS_IN_FLASH */


#ifdef CFG_REDUNDAND_ENVIRONMENT
# define ENV_HEADER_SIZE	(sizeof(unsigned long) + 1)
#else
# define ENV_HEADER_SIZE	(sizeof(unsigned long))
#endif


#define ENV_SIZE (CFG_ENV_SIZE - ENV_HEADER_SIZE)

typedef	struct environment_s {
	unsigned long	crc;		/* CRC32 over data bytes	*/
#ifdef CFG_REDUNDAND_ENVIRONMENT
	unsigned char	flags;		/* active/obsolete flags	*/
#endif
	unsigned char	data[ENV_SIZE]; /* Environment data		*/
} env_t;

#ifdef CONFIG_BUFFALO_PLATFORM

#define CFG_ENV_B_VERSION			0x0006
#define CFG_ENV_B_SIZE				0x0400

#define DB_BUF_BOARD_MAX_MPP_CONFIG_NUM		0x1
#define DB_BUF_BOARD_MAX_DEVICE_CONFIG_NUM	0x4
#define DB_BUF_BOARD_MAX_PCI_IF_NUM		0x4
#define DB_BUF_BOARD_MAX_TWSI_DEF_NUM		0x4
#define DB_BUF_BOARD_MAX_MAC_INFO_NUM		0x4
#define DB_BUF_BOARD_MAX_GPP_INFO_NUM		0x4
#define DB_BUF_BOARD_MAX_DEBUG_LED_NUM		0x4

#define DB_BUF_BOARD_DEFAULT_MPP_CONFIG_NUM	0x1
#define DB_BUF_BOARD_DEFAULT_DEVICE_CONFIG_NUM	0x2
#define DB_BUF_BOARD_DEFAULT_PCI_IF_NUM		0x0
#define DB_BUF_BOARD_DEFAULT_TWSI_DEF_NUM	0x1
#define DB_BUF_BOARD_DEFAULT_MAC_INFO_NUM	0x1
#define DB_BUF_BOARD_DEFAULT_GPP_INFO_NUM	0x1
#define DB_BUF_BOARD_DEFAULT_DEBUG_LED_NUM	0x0

#define BUFFALO_BOARD_DEFAULT_MPP0_7		0x00000003
#define BUFFALO_BOARD_DEFAULT_MPP8_15		0x00000000
#define BUFFALO_BOARD_DEFAULT_MPP16_23		0x00005555
#define BUFFALO_BOARD_DEFAULT_MPP_DEV		0x03FF0000
#define BUFFALO_BOARD_DEFAULT_GPP_OE		0xFFC1BDF0
#define BUFFALO_BOARD_DEFAULT_GPP_VAL		0x00000000
#define BUFFALO_BOARD_DEFAULT_GPP_POLVAL	0x0040858D
#define BUFFALO_BOARD_DEFAULT_SERIES_NAME	"LinkStation"
#define BUFFALO_BOARD_DEFAULT_PRODUCT_NAME	"LS-QL(NINTOKU)"

#define USE_SWITCH_SLIDE_POWER			0	// 0:not use, 1:use

#define BUF_NV_ADDR_VERSION			0x0000
#define BUF_NV_ADDR_INTS_GPP_MASK		0x0008
#define BUF_NV_ADDR_NUM_BOARD_MPP		0x000C
#define BUF_NV_ADDR_NUM_BOARD_DEVICE_IF		0x000D
#define BUF_NV_ADDR_NUM_BOARD_PCI_IF		0x000E
#define BUF_NV_ADDR_NUM_BOARD_TWSI_DEV		0x000F
#define BUF_NV_ADDR_NUM_BOARD_MAC_INFO		0x0010
#define BUF_NV_ADDR_NUM_BOARD_GPP_INFO		0x0011
#define BUF_NV_ADDR_ACTIVE_LEDS_NUMBER		0x0012
#define BUF_NV_ADDR_LEDS_POLARITY		0x0013
#define BUF_NV_ADDR_GPP_OUT_EN_VAL		0x0014
#define BUF_NV_ADDR_GPP_OUT_VAL			0x0018
#define BUF_NV_ADDR_GPP_POLARITY_VAL		0x001C

#define BUF_NV_ADDR_BOARD_MPP_INFO		0x0040

#define BUF_NV_ADDR_DEV_CS_INFO			0x0070
#define BUF_NV_ADDR_DEV_CS_INFO_0		0x0070
#define BUF_NV_ADDR_DEV_CS_INFO_1		0x0078
#define BUF_NV_ADDR_DEV_CS_INFO_2		0x0080
#define BUF_NV_ADDR_DEV_CS_INFO_3		0x0088

#define BUF_NV_ADDR_BOARD_PCI_INFO		0x0090
#define BUF_NV_ADDR_BOARD_PCI_INFO_0		0x0090
#define BUF_NV_ADDR_BOARD_PCI_INFO_1		0x0098
#define BUF_NV_ADDR_BOARD_PCI_INFO_2		0x00A0
#define BUF_NV_ADDR_BOARD_PCI_INFO_3		0x00A8

#define BUF_NV_ADDR_BOARD_TWSI_INFO		0x00B0
#define BUF_NV_ADDR_BOARD_MAC_INFO		0x00C0
#define BUF_NV_ADDR_BOARD_GPP_INFO		0x00C8
#define BUF_NV_ADDR_LED_GPP_PIN			0x00D0

#define BUF_NV_ADDR_BIT_INPUT			0x00E0
#define BUF_NV_SIZE_BIT_INPUT			0x20
#define BUF_NV_OFFS_BIT_MICON			0x0
#define BUF_NV_OFFS_BIT_RTC			0x1
#define BUF_NV_OFFS_BIT_UPS			0x2
#define BUF_NV_OFFS_BIT_OMR_BL			0x3
#define BUF_NV_OFFS_BIT_FAN_LOCK		0x4
#define BUF_NV_OFFS_SW_POWER			0x5
#define BUF_NV_OFFS_SW_INIT			0x6
#define BUF_NV_OFFS_SW_RAID			0x7
#define BUF_NV_OFFS_SW_AUTO_POWER		0x8
#define BUF_NV_OFFS_SW_FUNC			0x9
#define BUF_NV_ADDR_BIT_MICON			BUF_NV_ADDR_BIT_INPUT + BUF_NV_OFFS_BIT_MICON
#define BUF_NV_ADDR_BIT_RTC			BUF_NV_ADDR_BIT_INPUT + BUF_NV_OFFS_BIT_RTC
#define BUF_NV_ADDR_BIT_UPS			BUF_NV_ADDR_BIT_INPUT + BUF_NV_OFFS_BIT_UPS
#define BUF_NV_ADDR_BIT_OMR_BL			BUF_NV_ADDR_BIT_INPUT + BUF_NV_OFFS_BIT_OMR_BL
#define BUF_NV_ADDR_BIT_FAN_LOCK		BUF_NV_ADDR_BIT_INPUT + BUF_NV_OFFS_BIT_FAN_LOCK
#define BUF_NV_ADDR_SW_POWER			BUF_NV_ADDR_BIT_INPUT + BUF_NV_OFFS_SW_POWER
#define BUF_NV_ADDR_SW_INIT			BUF_NV_ADDR_BIT_INPUT + BUF_NV_OFFS_SW_INIT
#define BUF_NV_ADDR_SW_RAID			BUF_NV_ADDR_BIT_INPUT + BUF_NV_OFFS_SW_RAID
#define BUF_NV_ADDR_SW_AUTO_POWER		BUF_NV_ADDR_BIT_INPUT + BUF_NV_OFFS_SW_AUTO_POWER
#define BUF_NV_ADDR_SW_FUNC			BUF_NV_ADDR_BIT_INPUT + BUF_NV_OFFS_SW_FUNC

#define BUF_NV_ADDR_BIT_HDD_POWER		BUF_NV_ADDR_BIT_INPUT + BUF_NV_SIZE_BIT_INPUT // 0x0100
#define BUF_NV_SIZE_BIT_HDD_POWER		0x10

#define BUF_NV_ADDR_BIT_USB_POWER		BUF_NV_ADDR_BIT_HDD_POWER + BUF_NV_SIZE_BIT_HDD_POWER // 0x0110
#define BUF_NV_SIZE_BIT_USB_POWER		0x04

#define BUF_NV_ADDR_BIT_OUTPUT			BUF_NV_ADDR_BIT_USB_POWER + BUF_NV_SIZE_BIT_USB_POWER // 0x0114
#define BUF_NV_SIZE_BIT_OUTPUT			0xC
#define BUF_NV_OFFS_BIT_FAN_LOW			0x0
#define BUF_NV_OFFS_BIT_FAN_HIGH		0x1
#define BUF_NV_OFFS_BIT_FAN_FULL		0x2
#define BUF_NV_OFFS_BIT_FAN_STOP		0x3
#define BUF_NV_ADDR_BIT_FAN_LOW			BUF_NV_ADDR_BIT_OUTPUT + BUF_NV_OFFS_BIT_FAN_LOW
#define BUF_NV_ADDR_BIT_FAN_HIGH		BUF_NV_ADDR_BIT_OUTPUT + BUF_NV_OFFS_BIT_FAN_HIGH
#define BUF_NV_ADDR_BIT_FAN_FULL		BUF_NV_ADDR_BIT_OUTPUT + BUF_NV_OFFS_BIT_FAN_FULL
#define BUF_NV_ADDR_BIT_FAN_STOP		BUF_NV_ADDR_BIT_OUTPUT + BUF_NV_OFFS_BIT_FAN_STOP

#define BUF_NV_ADDR_BIT_LED			BUF_NV_ADDR_BIT_OUTPUT + BUF_NV_SIZE_BIT_OUTPUT // 0x0120
#define BUF_NV_SIZE_BIT_LED			0x10
#define BUF_NV_OFFS_BIT_LED_POWER		0x0
#define BUF_NV_OFFS_BIT_LED_INFO		0x1
#define BUF_NV_OFFS_BIT_LED_ALARM		0x2
#define BUF_NV_OFFS_BIT_LED_FUNC		0x3
#define BUF_NV_OFFS_BIT_LED_ETH			0x4
#define BUF_NV_ADDR_BIT_LED_POWER		BUF_NV_ADDR_BIT_LED + BUF_NV_OFFS_BIT_LED_POWER
#define BUF_NV_ADDR_BIT_LED_INFO		BUF_NV_ADDR_BIT_LED + BUF_NV_OFFS_BIT_LED_INFO
#define BUF_NV_ADDR_BIT_LED_ALARM		BUF_NV_ADDR_BIT_LED + BUF_NV_OFFS_BIT_LED_ALARM
#define BUF_NV_ADDR_BIT_LED_FUNC		BUF_NV_ADDR_BIT_LED + BUF_NV_OFFS_BIT_LED_FUNC
#define BUF_NV_ADDR_BIT_LED_ETH			BUF_NV_ADDR_BIT_LED + BUF_NV_OFFS_BIT_LED_ETH

#define BUF_NV_ADDR_LED_GRP			BUF_NV_ADDR_BIT_LED + BUF_NV_SIZE_BIT_LED // 0x0130
#define BUF_NV_ADDR_LED_GRP_POWER		BUF_NV_ADDR_LED_GRP + BUF_NV_OFFS_BIT_LED_POWER
#define BUF_NV_ADDR_LED_GRP_INFO		BUF_NV_ADDR_LED_GRP + BUF_NV_OFFS_BIT_LED_INFO
#define BUF_NV_ADDR_LED_GRP_ALARM		BUF_NV_ADDR_LED_GRP + BUF_NV_OFFS_BIT_LED_ALARM
#define BUF_NV_ADDR_LED_GRP_FUNC		BUF_NV_ADDR_LED_GRP + BUF_NV_OFFS_BIT_LED_FUNC
#define BUF_NV_ADDR_LED_GRP_ETH			BUF_NV_ADDR_LED_GRP + BUF_NV_OFFS_BIT_LED_ETH

#define BUF_NV_ADDR_USE_SWITCH_SLIDE_POWER	0x0140

#define BUF_NV_ADDR_PID				0x0204
#define BUF_NV_ADDR_SERIES_NAME			0x0210
#define BUF_NV_SIZE_SERIES_NAME			0x0020

#define BUF_NV_ADDR_PRODUCT_NAME		BUF_NV_ADDR_SERIES_NAME + BUF_NV_SIZE_SERIES_NAME // 0x0230
#define BUF_NV_SIZE_PRODUCT_NAME		0x20

#endif // end of CONFIG_BUFFALO_PLATFORM

#endif	/* _ENVIRONMENT_H_ */
